#!/usr/bin/env ts-node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const flash_store_1 = require("flash-store");
const config_1 = require("../src/config");
let cacheContactRawPayload;
let cacheRoomRawPayload;
let cacheRoomMemberRawPayload;
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const workdir = process.env.STORE_HOME;
        if (!workdir) {
            config_1.log.info('Dumper', 'main() Usage: `STORE_HOME=xxx dumper.ts');
            throw new Error('STORE_HOME env var not set');
        }
        if (!(yield fs_extra_1.default.pathExists(workdir))) {
            throw new Error('path not exist: ' + workdir);
        }
        cacheContactRawPayload = new flash_store_1.FlashStoreSync(path_1.default.join(workdir, 'contact-raw-payload'));
        cacheRoomRawPayload = new flash_store_1.FlashStoreSync(path_1.default.join(workdir, 'room-raw-payload'));
        cacheRoomMemberRawPayload = new flash_store_1.FlashStoreSync(path_1.default.join(workdir, 'room-member-raw-payload'));
        yield Promise.all([
            cacheContactRawPayload.ready(),
            cacheRoomRawPayload.ready(),
            cacheRoomMemberRawPayload.ready(),
        ]);
        const roomMemberTotalNum = [...cacheRoomMemberRawPayload.values()].reduce((accuVal, currVal) => {
            return accuVal + Object.keys(currVal).length;
        }, 0);
        config_1.log.warn('Dumper', 'main() Store status: contact: %d, room: %d, room members: %d', cacheContactRawPayload.size, cacheRoomRawPayload.size, roomMemberTotalNum);
        dumpHtml();
    });
}
function dumpHtml() {
    console.log(`
    <html>
    <head>
      <style>
        table, th, td {
          border: 1px solid black;
        }
      </style>
    </head>
    <body>
  `);
    // dumpRooms()
    dumpRoomMembers();
    // dumpContacts()
    console.log(`
    </body>
    </html>
  `);
}
/**
 * Cotnacts
 */
function dumpContacts() {
    let n = 0;
    console.log(`
    <h2>Contacts</h2>
    <table>
      <tr>
        <td>
          #
        </td>
        <th>
          Profile Photo
        </th>
        <th>
          Wechat Name
        </th>
      </tr>
  `);
    for (const payload of cacheContactRawPayload.values()) {
        if (!payload.user_name) {
            continue;
        }
        console.log(`
      <tr>
        <td>
          ${++n}
        </td>
        <td>
          <image width="40" height="40" src=${payload.small_head} />
        </td>
        <td>
          ${payload.nick_name}
        </td>
      </tr>
    `);
    }
    console.log(`
    </table>
  `);
}
exports.dumpContacts = dumpContacts;
/**
 * Rooms
 */
function dumpRooms() {
    let n = 0;
    console.log(`
    <h2>Rooms</h2>
    <table>
      <tr>
        <td>
          #
        </td>
        <th>
          Room Topic
        </th>
        <th>
          Members Number
        </th>
      </tr>
  `);
    for (const payload of cacheRoomRawPayload.values()) {
        if (!payload.user_name) {
            continue;
        }
        console.log(`
      <tr>
        <td>
          ${++n}
        </td>
        <td>
          ${payload.nick_name}
        </td>
        <td>
          ${payload.member_count}
        </td>
      </tr>
    `);
    }
    console.log(`
    </table>
  `);
}
exports.dumpRooms = dumpRooms;
/**
 * Room Members
 */
function dumpRoomMembers() {
    let n = 0;
    console.log(`
  <h2>Room Members</h2>
  <table>
    <tr>
      <th>
        #
      </th>
      <th>
        Room Topic
      </th>
      <th>
        Member Photo
      </th>
      <th>
        Member Name
      </th>
    </tr>
  `);
    for (const [roomid, memberDictPayload] of cacheRoomMemberRawPayload) {
        const roomPayload = cacheRoomRawPayload.get(roomid);
        if (!roomPayload) {
            continue;
        }
        for (const memberWxid of Object.keys(memberDictPayload)) {
            const memberPayload = memberDictPayload[memberWxid];
            if (!memberPayload.user_name) {
                continue;
            }
            console.log(`
        <tr>
          <td>
            ${++n}
          </td>
          <td>
            ${roomPayload.nick_name}
          </td>
          <td>
            <img width="40" height="40" src="${memberPayload.small_head}" />
          </td>
          <td>
            ${memberPayload.nick_name}
          </td>
        </tr>
      `);
        }
    }
}
exports.dumpRoomMembers = dumpRoomMembers;
main()
    .catch(console.error);
//# sourceMappingURL=store-dumper.js.map