import { MemoryCard } from 'memory-card';
import { PadchatContactPayload, PadchatRoomInvitationPayload, PadchatRoomInviteEvent, PadchatRoomMemberPayload, PadchatRoomPayload } from './padchat-schemas';
import { PadchatRpc } from './padchat-rpc';
export interface PadchatMemorySlot {
    device: {
        [userId: string]: undefined | {
            data: string;
            token: string;
        };
    };
    currentUserId?: string;
}
export interface ManagerOptions {
    memory: MemoryCard;
    endpoint: string;
    token: string;
}
export declare class PadchatManager extends PadchatRpc {
    options: ManagerOptions;
    private memorySlot;
    private loginScanQrcode?;
    private loginScanStatus?;
    private loginScanTimer?;
    private cacheContactRawPayload?;
    private cacheRoomMemberRawPayload?;
    private cacheRoomRawPayload?;
    private cacheRoomInvitationRawPayload?;
    private readonly state;
    private readonly delayQueueExecutor;
    private delayQueueExecutorSubscription?;
    private roomNeedsToBeSync;
    private contactListSynced;
    constructor(options: ManagerOptions);
    protected initCache(token: string, userId: string): Promise<void>;
    protected releaseCache(): Promise<void>;
    start(): Promise<void>;
    reconnect(): Promise<void>;
    stop(): Promise<void>;
    protected onLogin(userId: string): Promise<void>;
    logout(): Promise<void>;
    protected stopCheckScan(): Promise<void>;
    protected startCheckScan(): Promise<void>;
    /**
     * Offline, then relogin
     * emit qrcode or send login request to the user.
     */
    protected tryAutoLogin(memorySlot: PadchatMemorySlot): Promise<boolean>;
    protected emitLoginQrcode(): Promise<void>;
    protected refreshMemorySlotData(memorySlot: PadchatMemorySlot, userId: string): Promise<PadchatMemorySlot>;
    protected tryLoad62Data(): Promise<void>;
    getContactIdList(): string[];
    getRoomIdList(): string[];
    roomMemberRawPayloadDirty(roomId: string): void;
    getRoomMemberIdList(roomId: string, dirty?: boolean): Promise<string[]>;
    roomRawPayloadDirty(roomId: string): void;
    roomMemberRawPayload(roomId: string): Promise<{
        [contactId: string]: PadchatRoomMemberPayload;
    }>;
    syncRoomMember(roomId: string): Promise<{
        [contactId: string]: PadchatRoomMemberPayload;
    }>;
    syncContactsAndRooms(): Promise<void>;
    contactRawPayloadDirty(contactId: string): void;
    contactRawPayload(contactId: string): Promise<PadchatContactPayload>;
    roomRawPayload(id: string): Promise<PadchatRoomPayload>;
    roomInvitationRawPayload(roomInvitationId: string): Promise<PadchatRoomInvitationPayload>;
    roomInvitationRawPayloadDirty(roomInvitationId: string): Promise<void>;
    saveRoomInvitationRawPayload(roomInvitation: PadchatRoomInviteEvent): Promise<void>;
    ding(data?: string): void;
    updateSelfName(newName: string): Promise<void>;
    updateSelfSignature(signature: string): Promise<void>;
}
export default PadchatManager;
