#!/usr/bin/env ts-node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const blue_tape_1 = __importDefault(require("blue-tape"));
// import sinon from 'sinon'
const memory_card_1 = require("memory-card");
const config_1 = require("./config");
const padchat_manager_1 = require("./padchat-manager");
class PadchatManagerTest extends padchat_manager_1.PadchatManager {
    onSocket(payload) {
        return super.onSocket(payload);
    }
    initCache(token, selfId) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            return _super("initCache").call(this, token, selfId);
        });
    }
    releaseCache() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            return _super("releaseCache").call(this);
        });
    }
}
blue_tape_1.default('PadchatManager() cache should be release and can be re-init again.', (t) => __awaiter(this, void 0, void 0, function* () {
    const manager = new PadchatManagerTest({
        endpoint: config_1.WECHATY_PUPPET_PADCHAT_ENDPOINT,
        memory: new memory_card_1.MemoryCard(),
        token: 'mock token',
    });
    try {
        for (let i = 0; i < 3; i++) {
            yield manager.initCache('test-fake-token', 'fake-self-id');
            yield manager.releaseCache();
            t.pass('init/release-ed at #' + i);
        }
        t.pass('PadchatManager() cache init/release/init successed.');
    }
    catch (e) {
        t.fail(e);
    }
}));
blue_tape_1.default('PadchatManager() cache release 10 instances for the same time', (t) => __awaiter(this, void 0, void 0, function* () {
    const MAX_NUM = 10;
    const managerList = [];
    for (let i = 0; i < MAX_NUM; i++) {
        const manager = new PadchatManagerTest({
            endpoint: config_1.WECHATY_PUPPET_PADCHAT_ENDPOINT,
            memory: new memory_card_1.MemoryCard(),
            token: 'mock token',
        });
        yield manager.initCache('test-fake-token-' + i, 'fake-self-id-' + i);
        managerList.push(manager);
    }
    try {
        yield Promise.all(managerList.map(manager => manager.releaseCache()));
        t.pass('release ' + MAX_NUM + ' at the same time success');
    }
    catch (e) {
        t.fail(e);
    }
}));
// TODO: Temporary disabled when launch token validation, enable me later
// test('PadchatManager() should can be able to restart() many times for one instance', async t => {
//   const MAX_NUM = 3
//   const memory = new MemoryCard()
//   await memory.load()
//   const manager = new PadchatManager({
//     endpoint : WECHATY_PUPPET_PADCHAT_ENDPOINT,
//     memory,
//     token    : 'test-mock-token',
//   })
//   try {
//     for (let i = 0; i < MAX_NUM; i++) {
//       await manager.start()
//       await manager.stop()
//       t.pass('restarted at #' + i)
//     }
//     t.pass('PadchatManager() restart successed.')
//   } catch (e) {
//     console.error(e)
//     t.fail(e)
//   }
// })
// TODO: Temporary disabled when launch token validation, enable me later
// test('PadchatManager() stop many instances for the same time', async t => {
//   const MAX_NUM = 3
//   const managerList = [] as PadchatManagerTest[]
//   const memory = new MemoryCard()
//   await memory.load()
//   const sandbox = sinon.createSandbox()
//   for (let i = 0; i < MAX_NUM; i++ ) {
//     const manager = new PadchatManagerTest({
//       endpoint : WECHATY_PUPPET_PADCHAT_ENDPOINT,
//       memory,
//       token    : 'mock token ' + i,
//     })
//     await manager.start()
//     managerList.push(manager)
//   }
//   const stopFutureList = [] as Array<Promise<void>>
//   for (let i = 0; i < MAX_NUM; i++) {
//     const manager = managerList[i]
//     const future = manager.stop()
//     stopFutureList.push(future)
//   }
//   try {
//     await Promise.all(stopFutureList)
//     t.pass('stop' + MAX_NUM + ' at the same time success')
//   } catch (e) {
//     t.fail(e)
//   }
//   sandbox.restore()
// })
//# sourceMappingURL=padchat-manager.spec.js.map