/**
 *   Wechaty - https://github.com/chatie/wechaty
 *
 *   @copyright 2016-2018 Huan LI <zixia@zixia.net>
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
import { FileBox } from 'file-box';
import { ContactPayload, FriendshipPayload, MessagePayload, Puppet, PuppetOptions, Receiver, RoomInvitationPayload, RoomMemberPayload, RoomPayload, UrlLinkPayload } from 'wechaty-puppet';
import { PadchatManager } from './padchat-manager';
import { PadchatContactPayload, PadchatMessagePayload, PadchatRoomInvitationPayload, PadchatRoomMemberPayload, PadchatRoomPayload } from './padchat-schemas';
export declare class PuppetPadchat extends Puppet {
    options: PuppetOptions;
    static readonly VERSION: string;
    private padchatCounter;
    private readonly cachePadchatMessagePayload;
    private padchatManager?;
    constructor(options?: PuppetOptions);
    toString(): string;
    ding(data?: string): void;
    startWatchdog(): void;
    start(): Promise<void>;
    protected login(selfId: string): Promise<void>;
    startManager(manager: PadchatManager): Promise<void>;
    protected onPadchatMessage(rawPayload: PadchatMessagePayload): Promise<void>;
    protected onPadchatMessageRoomInvitation(rawPayload: PadchatMessagePayload): Promise<void>;
    /**
     * Look for room join event
     */
    protected onPadchatMessageRoomEventJoin(rawPayload: PadchatMessagePayload): Promise<void>;
    /**
     * Look for room leave event
     */
    protected onPadchatMessageRoomEventLeave(rawPayload: PadchatMessagePayload): Promise<void>;
    /**
     * Look for room topic event
     */
    protected onPadchatMessageRoomEventTopic(rawPayload: PadchatMessagePayload): Promise<void>;
    protected onPadchatMessageFriendshipEvent(rawPayload: PadchatMessagePayload): Promise<void>;
    stop(): Promise<void>;
    logout(shallow?: boolean): Promise<void>;
    /**
     *
     * Contact
     *
     */
    contactAlias(contactId: string): Promise<string>;
    contactAlias(contactId: string, alias: string | null): Promise<void>;
    contactValidate(contactId: string): Promise<boolean>;
    contactList(): Promise<string[]>;
    contactAvatar(contactId: string): Promise<FileBox>;
    contactAvatar(contactId: string, file: FileBox): Promise<void>;
    contactSelfQrcode(): Promise<string>;
    private getQRCode;
    contactPayloadDirty(contactId: string): Promise<void>;
    contactRawPayload(contactId: string): Promise<PadchatContactPayload>;
    contactRawPayloadParser(rawPayload: PadchatContactPayload): Promise<ContactPayload>;
    /**
     * Overwrite the Puppet.contactPayload()
     */
    contactPayload(contactId: string): Promise<ContactPayload>;
    /**
     *
     * Message
     *
     */
    messageFile(messageId: string): Promise<FileBox>;
    messageUrl(messageId: string): Promise<UrlLinkPayload>;
    private getVoiceFileBoxFromRawPayload;
    private getVoiceDataFromRawPayload;
    messageRawPayload(id: string): Promise<PadchatMessagePayload>;
    messageRawPayloadParser(rawPayload: PadchatMessagePayload): Promise<MessagePayload>;
    messageSendText(receiver: Receiver, text: string, mentionIdList?: string[]): Promise<void>;
    messageSendFile(receiver: Receiver, file: FileBox): Promise<void>;
    messageSendContact(receiver: Receiver, contactId: string): Promise<void>;
    messageSendUrl(receiver: Receiver, urlLinkPayload: UrlLinkPayload): Promise<void>;
    private forwardAttachment;
    messageForward(receiver: Receiver, messageId: string): Promise<void>;
    /**
     *
     * Room
     *
     */
    roomMemberPayloadDirty(roomId: string): Promise<void>;
    roomMemberRawPayload(roomId: string, contactId: string): Promise<PadchatRoomMemberPayload>;
    roomMemberRawPayloadParser(rawPayload: PadchatRoomMemberPayload): Promise<RoomMemberPayload>;
    roomPayloadDirty(roomId: string): Promise<void>;
    roomRawPayload(roomId: string): Promise<PadchatRoomPayload>;
    roomRawPayloadParser(rawPayload: PadchatRoomPayload): Promise<RoomPayload>;
    roomMemberList(roomId: string): Promise<string[]>;
    roomValidate(roomId: string): Promise<boolean>;
    roomList(): Promise<string[]>;
    roomDel(roomId: string, contactId: string): Promise<void>;
    roomQrcode(roomId: string): Promise<string>;
    roomAvatar(roomId: string): Promise<FileBox>;
    roomAdd(roomId: string, contactId: string): Promise<void>;
    roomTopic(roomId: string): Promise<string>;
    roomTopic(roomId: string, topic: string): Promise<void>;
    roomCreate(contactIdList: string[], topic: string): Promise<string>;
    roomQuit(roomId: string): Promise<void>;
    roomAnnounce(roomId: string): Promise<string>;
    roomAnnounce(roomId: string, text: string): Promise<void>;
    roomInvitationRawPayload(roomInvitationId: string): Promise<PadchatRoomInvitationPayload>;
    roomInvitationRawPayloadParser(rawPayload: PadchatRoomInvitationPayload): Promise<RoomInvitationPayload>;
    roomInvitationAccept(roomInvitationId: string): Promise<void>;
    /**
     *
     * Friendship
     *
     */
    friendshipAdd(contactId: string, hello: string): Promise<void>;
    friendshipAccept(friendshipId: string): Promise<void>;
    friendshipRawPayloadParser(rawPayload: PadchatMessagePayload): Promise<FriendshipPayload>;
    friendshipRawPayload(friendshipId: string): Promise<PadchatMessagePayload>;
    unref(): void;
    contactSelfName(newName: string): Promise<void>;
    contactSelfSignature(signature: string): Promise<void>;
}
export default PuppetPadchat;
