#!/usr/bin/env ts-node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const blue_tape_1 = __importDefault(require("blue-tape"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const puppet_padchat_1 = require("../src/puppet-padchat");
const mock_server_1 = require("./mock-server");
const mock_config_1 = require("./mock-config");
const TOTAL_TEST_CHECK = 2;
const clearFlashStoreCache = () => __awaiter(this, void 0, void 0, function* () {
    const baseDir = path_1.default.join(os_1.default.homedir(), path_1.default.sep, '.wechaty', 'puppet-padchat-cache', path_1.default.sep, mock_config_1.MOCK_TOKEN, path_1.default.sep, mock_config_1.MOCK_USER_ID);
    yield fs_extra_1.default.remove(baseDir);
});
blue_tape_1.default('PuppetPadchat Integration test', (t) => __awaiter(this, void 0, void 0, function* () {
    yield clearFlashStoreCache();
    const mockServer = new mock_server_1.MockSocketServer(mock_config_1.FAKE_SERVER_PORT);
    mockServer.setContactPayloadMockArray([]);
    yield mockServer.start();
    const puppet = new puppet_padchat_1.PuppetPadchat({
        token: mock_config_1.MOCK_TOKEN,
        endpoint: `http://localhost:${mock_config_1.FAKE_SERVER_PORT}`
    });
    yield puppet.start();
    puppet.on('login', () => __awaiter(this, void 0, void 0, function* () {
        const validateResult = yield puppet.contactValidate('id');
        t.ok(!validateResult, 'validate result should be false');
        const contactList = yield puppet.contactList();
        t.ok(contactList.length === 1, 'contact list should be 1, which only contains self');
        yield puppet.stop();
        t.plan(TOTAL_TEST_CHECK);
        yield mockServer.stop();
    }));
}));
//# sourceMappingURL=integration.spec.js.map