#!/usr/bin/env ts-node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const blue_tape_1 = __importDefault(require("blue-tape"));
const wechaty_puppet_1 = require("wechaty-puppet");
const padchat_schemas_1 = require("../padchat-schemas");
const contact_raw_payload_parser_1 = require("./contact-raw-payload-parser");
blue_tape_1.default('contactRawPayloadParser', (t) => __awaiter(this, void 0, void 0, function* () {
    const PADCHAT_CONTACT_PAYLOAD_PERSONAL = {
        big_head: 'http://wx.qlogo.cn/mmhead/ver_1/xfCMmibHH74xGLoyeDFJadrZXX3eOEznPefiaCa3iczxZGMwPtDuSbRQKx3Xdm18un303mf0NFia3USY2nO2VEYILw/0',
        city: 'Haidian',
        continue: padchat_schemas_1.PadchatContinue.Done,
        country: 'CN',
        intro: '',
        label: '1',
        msg_type: padchat_schemas_1.PadchatContactMsgType.Contact,
        nick_name: '梦君君',
        provincia: 'Beijing',
        py_initial: 'LJR',
        remark: '女儿',
        remark_py_initial: 'lijiaruibeizhu',
        remark_quan_pin: 'LJRBZ',
        sex: wechaty_puppet_1.ContactGender.Female,
        signature: 'Stay+Foolish',
        small_head: 'http://wx.qlogo.cn/mmhead/ver_1/xfCMmibHH74xGLoyeDFJadrZXX3eOEznPefiaCa3iczxZGMwPtDuSbRQKx3Xdm18un303mf0NFia3USY2nO2VEYILw/132',
        status: padchat_schemas_1.PadchatContactRoomStatus.Get,
        stranger: 'v1_0468f2cd3f0efe7ca2589d57c3f9ba952a3789e41b6e78ee00ed53d1e6096b88@stranger',
        ticket: '',
        user_name: 'mengjunjun001',
    };
    const PADCHAT_CONTACT_PAYLOAD_OFFICIAL = {
        big_head: 'http://wx.qlogo.cn/mmhead/ver_1/TR8EDh3MgMsu20pxjrDPBpaGySuEAGf3MUuoeUOV2LiaqvZxeMqb1U7hgiciaQZBC8LYN0boVLCKOIYg71pxdl1fQabiaxsn7CnNeGWVrK3jSIY/0',
        city: 'Haidian',
        country: 'CN',
        intro: 'CARPE+DIEM+-+if+not+us,+who?+if+not+now,+when?',
        label: '',
        message: '',
        nick_name: '李卓桓',
        provincia: 'Beijing',
        py_initial: 'LZH',
        quan_pin: 'lizhuohuan',
        remark: '',
        remark_py_initial: '',
        remark_quan_pin: '',
        sex: 0,
        signature: 'CARPE+DIEM+-+if+not+us,+who?+if+not+now,+when?',
        small_head: 'http://wx.qlogo.cn/mmhead/ver_1/TR8EDh3MgMsu20pxjrDPBpaGySuEAGf3MUuoeUOV2LiaqvZxeMqb1U7hgiciaQZBC8LYN0boVLCKOIYg71pxdl1fQabiaxsn7CnNeGWVrK3jSIY/132',
        status: 0,
        stranger: 'v1_cd6656d42f505e5ffbb7eab65fed448fc8f02eade29a873ec3e758c7553db424@stranger',
        ticket: '',
        user_name: 'gh_59d7c8ad720c',
    };
    const EXPECTED_CONTACT_PAYLOAD_PERSONAL = {
        alias: '女儿',
        avatar: 'http://wx.qlogo.cn/mmhead/ver_1/xfCMmibHH74xGLoyeDFJadrZXX3eOEznPefiaCa3iczxZGMwPtDuSbRQKx3Xdm18un303mf0NFia3USY2nO2VEYILw/0',
        city: 'Haidian',
        gender: wechaty_puppet_1.ContactGender.Female,
        id: 'mengjunjun001',
        name: '梦君君',
        province: 'Beijing',
        signature: 'Stay Foolish',
        type: wechaty_puppet_1.ContactType.Personal,
    };
    const EXPECTED_CONTACT_PAYLOAD_OFFICIAL = {
        alias: '',
        avatar: 'http://wx.qlogo.cn/mmhead/ver_1/TR8EDh3MgMsu20pxjrDPBpaGySuEAGf3MUuoeUOV2LiaqvZxeMqb1U7hgiciaQZBC8LYN0boVLCKOIYg71pxdl1fQabiaxsn7CnNeGWVrK3jSIY/0',
        city: 'Haidian',
        gender: wechaty_puppet_1.ContactGender.Unknown,
        id: 'gh_59d7c8ad720c',
        name: '李卓桓',
        province: 'Beijing',
        signature: 'CARPE DIEM+-+if+not+us,+who?+if+not+now,+when?',
        type: wechaty_puppet_1.ContactType.Official,
    };
    const resultPersonal = contact_raw_payload_parser_1.contactRawPayloadParser(PADCHAT_CONTACT_PAYLOAD_PERSONAL);
    t.deepEqual(resultPersonal, EXPECTED_CONTACT_PAYLOAD_PERSONAL, 'should parse ContactPayload for personal account payload');
    const resultOfficial = contact_raw_payload_parser_1.contactRawPayloadParser(PADCHAT_CONTACT_PAYLOAD_OFFICIAL);
    t.deepEqual(resultOfficial, EXPECTED_CONTACT_PAYLOAD_OFFICIAL, 'should parse ContactPayload for official account payload');
    t.throws(() => contact_raw_payload_parser_1.contactRawPayloadParser({}), 'should throw exception for invalid object');
    t.throws(() => contact_raw_payload_parser_1.contactRawPayloadParser(undefined), 'should throw exception for undifined');
}));
//# sourceMappingURL=contact-raw-payload-parser.spec.js.map