"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wechaty_puppet_1 = require("wechaty-puppet");
const _1 = require(".");
const padchat_schemas_1 = require("../padchat-schemas");
const is_type_1 = require("./is-type");
const message_file_name_1 = require("./message-file-name");
const message_source_parser_1 = require("./message-source-parser");
const message_type_1 = require("./message-type");
function messageRawPayloadParser(rawPayload) {
    return __awaiter(this, void 0, void 0, function* () {
        // console.log('messageRawPayloadParser:', rawPayload)
        /**
         * 0. Set Message Type
         */
        const type = message_type_1.messageType(rawPayload.sub_type);
        const payloadBase = {
            id: rawPayload.msg_id,
            timestamp: rawPayload.timestamp,
            type,
        };
        if (type === wechaty_puppet_1.MessageType.Image
            || type === wechaty_puppet_1.MessageType.Audio
            || type === wechaty_puppet_1.MessageType.Video
            || type === wechaty_puppet_1.MessageType.Attachment) {
            payloadBase.filename = message_file_name_1.messageFileName(rawPayload) || undefined;
        }
        let fromId;
        let roomId;
        let toId;
        let text;
        let mentionIdList;
        /**
         * 1. Set Room Id
         */
        if (is_type_1.isRoomId(rawPayload.from_user)) {
            roomId = rawPayload.from_user;
        }
        else if (is_type_1.isRoomId(rawPayload.to_user)) {
            roomId = rawPayload.to_user;
        }
        else {
            roomId = undefined;
        }
        /**
         * 2. Set To Contact Id
         */
        if (is_type_1.isContactId(rawPayload.to_user)) {
            toId = rawPayload.to_user;
        }
        else {
            // TODO: if the message @someone, the toId should set to the mentioned contact id(?)
            toId = undefined;
        }
        /**
         * 3. Set From Contact Id
         */
        if (is_type_1.isContactId(rawPayload.from_user)) {
            fromId = rawPayload.from_user;
        }
        else {
            const parts = rawPayload.content.split(':\n');
            if (parts && parts.length > 1) {
                if (is_type_1.isContactId(parts[0])) {
                    fromId = parts[0];
                }
            }
            else {
                fromId = undefined;
            }
        }
        /**
         *
         * 4. Set Text
         */
        if (is_type_1.isRoomId(rawPayload.from_user)) {
            const parts = rawPayload.content.split(':\n');
            if (parts && parts.length > 1) {
                text = parts[1];
            }
            else {
                text = rawPayload.content;
            }
        }
        else {
            text = rawPayload.content;
        }
        /**
         * 5.1 Validate Room & From ID
         */
        if (!roomId && !fromId) {
            throw Error('empty roomId and empty fromId!');
        }
        /**
         * 5.1 Validate Room & To ID
         */
        if (!roomId && !toId) {
            throw Error('empty roomId and empty toId!');
        }
        /**
         * 6. Set mention list, only for room messages
         */
        if (roomId) {
            const messageSource = yield message_source_parser_1.messageSourceParser(rawPayload.msg_source);
            if (messageSource !== null && messageSource.atUserList) {
                mentionIdList = messageSource.atUserList;
            }
        }
        /**
         * 6. Set Contact for ShareCard
         */
        // if (type === MessageType.Contact) {
        //   interface XmlSchema {
        //     msg: {
        //       username: string,
        //       bigheadimgurl: string,
        //       nickname: string,
        //       province: string,
        //       city: string,
        //       sign: string,
        //       sex: number,
        //       antispamticket: string,
        //     },
        //     t: PadchatContactPayload,
        //   }
        //   const jsonPayload = JSON.parse(toJson(text)) as XmlSchema
        //   console.log('jsonPayload:', jsonPayload)
        // }
        let payload;
        // Two branch is the same code.
        // Only for making TypeScript happy
        if (fromId && toId) {
            payload = Object.assign({}, payloadBase, { fromId,
                mentionIdList,
                roomId,
                text,
                toId });
        }
        else if (roomId) {
            payload = Object.assign({}, payloadBase, { fromId,
                mentionIdList,
                roomId,
                text,
                toId });
        }
        else {
            throw new Error('neither toId nor roomId');
        }
        if (type === wechaty_puppet_1.MessageType.Attachment) {
            const appPayload = yield _1.appMessageParser(rawPayload);
            if (appPayload) {
                switch (appPayload.type) {
                    case padchat_schemas_1.WechatAppMessageType.Url:
                        payload.type = wechaty_puppet_1.MessageType.Url;
                        break;
                    case padchat_schemas_1.WechatAppMessageType.Attach:
                        payload.type = wechaty_puppet_1.MessageType.Attachment;
                        break;
                    case padchat_schemas_1.WechatAppMessageType.ChatHistory:
                        payload.type = wechaty_puppet_1.MessageType.ChatHistory;
                        break;
                    case padchat_schemas_1.WechatAppMessageType.MiniProgram:
                        payload.type = wechaty_puppet_1.MessageType.MiniProgram;
                        break;
                    case padchat_schemas_1.WechatAppMessageType.RedEnvelopes:
                    case padchat_schemas_1.WechatAppMessageType.Transfers:
                        payload.type = wechaty_puppet_1.MessageType.Money;
                        break;
                    case padchat_schemas_1.WechatAppMessageType.RealtimeShareLocation:
                        payload.type = wechaty_puppet_1.MessageType.Location;
                        break;
                    default:
                        payload.type = wechaty_puppet_1.MessageType.Unknown;
                        break;
                }
            }
        }
        return payload;
    });
}
exports.messageRawPayloadParser = messageRawPayloadParser;
//# sourceMappingURL=message-raw-payload-parser.js.map